import ProjectDescription

public extension Project {
    static func app(
        name: String,
        packages: [Package],
        baseSettings: SettingsDictionary,
        infoPlist: InfoPlist,
        scripts: [TargetScript],
        entitlements: Path?,
        dependencies: [TargetDependency]
    ) -> Self {
        return Project(
            name: name,
            organizationName: "DevYeom.com",
            options: .options(),
            packages: packages,
            settings: .settings(
                base: baseSettings,
                configurations: .project,
                defaultSettings: .recommended
            ),
            targets: .appTargets(
                name: name,
                infoPlist: infoPlist,
                entitlements: entitlements,
                dependencies: dependencies
            ),
            schemes: [
                .debug(name: name),
                .beta(name: name),
                .release(name: name),
            ],
            fileHeaderTemplate: nil,
            additionalFiles: [],
            resourceSynthesizers: [
                .assets(),
                .strings(),
            ]
        )
    }

    static func dynamicFramework(
        name: String,
        packages: [Package],
        baseSettings: SettingsDictionary,
        dependencies: [TargetDependency],
        includeExample: Bool
    ) -> Self {
        return module(
            name: name,
            packages: packages,
            settings: .settings(
                base: baseSettings,
                configurations: .project,
                defaultSettings: .recommended
            ),
            product: .framework,
            dependencies: dependencies,
            includeExample: includeExample
        )
    }

    static func staticFramework(
        name: String,
        packages: [Package],
        baseSettings: SettingsDictionary,
        dependencies: [TargetDependency],
        includeExample: Bool
    ) -> Self {
        return module(
            name: name,
            packages: packages,
            settings: .settings(
                base: baseSettings,
                configurations: .project,
                defaultSettings: .recommended
            ),
            product: .staticFramework,
            dependencies: dependencies,
            includeExample: includeExample
        )
    }

    private static func module(
        name: String,
        packages: [Package],
        settings: Settings?,
        product: Product,
        dependencies: [TargetDependency],
        includeExample: Bool
    ) -> Self {
        return Project(
            name: name,
            organizationName: nil,
            options: .options(),
            packages: packages,
            settings: settings,
            targets: .targets(
                name: name,
                product: product,
                dependencies: dependencies,
                includeExample: includeExample
            ),
            schemes: [
                .debug(name: name),
                .beta(name: name),
                .release(name: name),
            ],
            fileHeaderTemplate: nil,
            additionalFiles: [],
            resourceSynthesizers: .default
        )
    }
}
