import ProjectDescription

extension Array where Element == Target {
    static func appTargets(
        name: String,
        infoPlist: InfoPlist,
        entitlements: Path?,
        dependencies: [TargetDependency]
    ) -> [Target] {
        return [
            .app(
                name: name,
                infoPlist: infoPlist,
                entitlements: entitlements,
                dependencies: dependencies
            ),
            .unitTests(
                name: name
            )
        ]
    }

    static func targets(
        name: String,
        product: Product,
        dependencies: [TargetDependency],
        includeExample: Bool
    ) -> [Target] {
        var targets: [Target] = [
            .target(
                name: name,
                product: product,
                dependencies: dependencies
            ),
            .unitTests(
                name: name
            )
        ]

        if includeExample {
            targets.append(.example(name: name))
        }

        return targets
    }
}

private extension Target {
    static func app(
        name: String,
        infoPlist: InfoPlist,
        entitlements: Path?,
        dependencies: [TargetDependency]
    ) -> Self {
        return Target(
            name: name,
            platform: .iOS,
            product: .app,
            productName: name,
            bundleId: "$(CUSTOM_BUNDLE_IDENTIFIER)",
            deploymentTarget: .app,
            infoPlist: infoPlist,
            sources: ["Sources/**"],
            resources: ["Resources/**"],
            copyFiles: nil,
            headers: nil,
            entitlements: entitlements,
            scripts: [],
            dependencies: dependencies,
            settings: .common(),
            coreDataModels: [],
            environment: [:],
            launchArguments: [],
            additionalFiles: []
        )
    }

    static func target(
        name: String,
        product: Product,
        dependencies: [TargetDependency]
    ) -> Self {
        return Target(
            name: name,
            platform: .iOS,
            product: product,
            productName: name,
            bundleId: "com.DevYeom.\(name)",
            deploymentTarget: .app,
            infoPlist: .default,
            sources: ["Sources/**"],
            resources: ["Resources/**"],
            copyFiles: nil,
            headers: nil,
            entitlements: nil,
            scripts: [],
            dependencies: dependencies,
            settings: .common(),
            coreDataModels: [],
            environment: [:],
            launchArguments: [],
            additionalFiles: []
        )
    }

    static func unitTests(
        name: String
    ) -> Self {
        return testTarget(
            name: name,
            product: .unitTests
        )
    }

    static func testTarget(
        name: String,
        product: Product
    ) -> Self {
        return Target(
            name: "\(name)Tests",
            platform: .iOS,
            product: product,
            productName: nil,
            bundleId: "com.DevYeom.\(name)Tests",
            deploymentTarget: .app,
            infoPlist: .default,
            sources: ["Tests/**"],
            resources: [],
            copyFiles: nil,
            headers: nil,
            entitlements: nil,
            scripts: [],
            dependencies: [
                .xctest,
                .target(name: name),
            ],
            settings: .common(),
            coreDataModels: [],
            environment: [:],
            launchArguments: [],
            additionalFiles: []
        )
    }

    static func example(
        name: String
    ) -> Self {
        return Target(
            name: "\(name)Example",
            platform: .iOS,
            product: .app,
            productName: nil,
            bundleId: "com.DevYeom.\(name)Example",
            deploymentTarget: .app,
            infoPlist: .extendingDefault(with: [
                "UILaunchStoryboardName": "LaunchScreen",
            ]),
            sources: ["Example/Sources/**"],
            resources: ["Example/Resources/**"],
            copyFiles: nil,
            headers: nil,
            entitlements: nil,
            scripts: [],
            dependencies: [
                .target(name: name),
            ],
            settings: .settings(),
            coreDataModels: [],
            environment: [:],
            launchArguments: [],
            additionalFiles: []
        )
    }
}
